/*! \file *********************************************************************
 * \brief 				Ansteuer Funktionen fr Grafik display
 * - File:              grafik_display.c
 * - Compiler:          AVR-GCC
 * - Supported devices: ATxMega
 * - AppNote:            
 * \author              Falko Jahn 
 *****************************************************************************/

//#include <avr/interrupt.h>
//#include <avr/pgmspace.h>
//#include <stdlib.h>
//#include <stdarg.h>
//#include <ctype.h>
//#include <string.h>
#include <avr/io.h>
#include "grafik_display.h"
#include "spi_driver.h"
#include <util/delay.h>	

SPI_Master_t spiMaster;

/* Instantiate pointer to ssPort. */
PORT_t *ssPort = &PORTD;

TLCD lcd;

const char DISPLAY_INIT[] = 
	{
	DISPLAY_SCAN_DIR_NORMAL,
	DISPLAY_TOPVIEW,		
	DISPLAY_NORMAL,
	DISPLAY_BIAS_1_9,
	DISPLAY_POWER_LOW_POWER,
	(DISPLAY_VOLTAGE+DISPLAY_OUTPUT_VOLTAGE),
	DISPLAY_VOLUME_MODE_1,
	0x20,
	DISPLAY_INDICATOR_OFF,
	DISPLAY_INDICATOR_MODE_OFF,
	0xAF
	};




/*! @brief Byte an Display senden
 *
 *   - SS Leitung setzen 
 *   - A0 Leitung setzen anhand von Daten oder Befehl 
 * @version 1.0
 * @author Falko Jahn 
 * @param cmd_data 	Kennung (1=Datenbyte, 2=Befehl)
 * @param data		Datenbyte
 * @return nichts 
*/
void display_write_byte(unsigned cmd_data, unsigned char data) 
{
	SPI_PORT.OUTCLR = SPI_SS_bm;
	if(cmd_data == 0)
	{
		PORT_A0.OUTSET = PIN_A0_bm;
	}
	else
	{
		PORT_A0.OUTCLR = PIN_A0_bm;
	}

	SPI_MasterTransceiveByte(&spiMaster,data);

	

	SPI_PORT.OUTSET = SPI_SS_bm;
}


/*!@brief Display Initialisieren
 *
 * Sendet Befehlskette zum initialisieren 	
 * @version 1.0
 * @author Falko Jahn 
 * @return nichts 
*/
void display_init() 
{

	
	// Init SS pin as output 
	PORTD.DIRSET = SPI_SS_bm;

	// Set SS output to low. (No slave addressed). */
	PORTD.OUTCLR = SPI_SS_bm;



	// Initialize SPI master on port D. */
	SPI_MasterInit(&spiMaster,
	               &SPID,
	               &PORTD,
	               false,
	               SPI_MODE_0_gc,
	               SPI_INTLVL_OFF_gc,
	               false,
	               SPI_PRESCALER_DIV4_gc);
	
	
	
	
	
	
	//set outputs AO und RESET
	PORT_A0.DIRSET  = PIN_A0_bm;             
	PORT_RST.DIRSET = PIN_RST_bm;
  
	//Set SPI PORT
	SPI_PORT.OUTSET = SPI_SCK_bm| SPI_MOSI_bm | SPI_SS_bm;

	
  
    //Reset the Display Controller
	PORT_RST.OUTCLR = PIN_RST_bm;
	_delay_ms(1);                   
	PORT_RST.OUTSET = PIN_RST_bm;
	_delay_ms(1);  
	
	//sent 11 init commands to Display
	for (unsigned char tmp = 0;tmp < 11;tmp++)
	{
		display_write_byte(CMD,DISPLAY_INIT[tmp]);
	}

	

}





//*----------------------------------------------------------------------------
// Display aus Mirror-Buffer anzeigen
//*----------------------------------------------------------------------------
void Lcd_Show(void)
{
	volatile uint8_t x, y;


  	for(y = 0; y < 8; y++) 
	{
    
    	display_write_byte(1,y + 0xB0); 				// Row select
    	display_write_byte(1,0x10);     				// Column select high
    	display_write_byte(1,0x00);     				// Column select low

    	for(x = 0; x < 128; x++) 
		{ 
			display_write_byte(0,lcd.Mirror[x][y]);
		}
  	}

}

//*----------------------------------------------------------------------------
// Pixel setzen
// x = 0..128, y = 0..64
//*----------------------------------------------------------------------------
void Lcd_SetPixel(uint8_t x, uint8_t y)
{
  if((y >= LCD_HEIGHT) || (x >= LCD_WIDTH)) return;
  if(lcd.invers) {
    lcd.Mirror[x][y / 8] &= ~(1 << (y & 0x07));
  } else {
    lcd.Mirror[x][y / 8] |= (1 << (y & 0x07));
  }
}


//*----------------------------------------------------------------------------
// Display (Mirror) lschen
// danach unbedingt Lcd_Show() aufrufen
//*----------------------------------------------------------------------------
void Lcd_Clear(void)
{
  uint16_t i;
  uint8_t  color = 0x00;
  uint8_t  *ptr  = (uint8_t *)&lcd.Mirror;

  if(lcd.invers) { color = 0xFF; }
  for(i = 0; i < sizeof(lcd.Mirror); i++) { *ptr++ = color; }
}

//*----------------------------------------------------------------------------
// Horizontale Linie zeichnen
// x = 0..128, y = 0..64, w = Lnge
//*----------------------------------------------------------------------------
void Lcd_Hline(uint8_t x, uint8_t y, uint8_t w)
{
  uint8_t xp;

  for(xp = x; xp <= x + w - 1; xp++) { Lcd_SetPixel(xp, y); }
}
