/*----------------------------------------------------------------------------
 File:				grafik_display.h
 Author:         	Falko Jahn
 Version:        	05.03.2010
------------------------------------------------------------------------------*/

#ifndef _DISPLAY_H

	#define _DISPLAY_H

	typedef struct 
	{
		uint8_t   	Mirror[128][8];    // Display Mirror-Buffer 4 x 132 Bytes
		uint8_t 	invers;
	} TLCD;


	

	#define LCD_HEIGHT				64
	#define LCD_WIDTH				128
	#define DISPLAY_OUTPUT_VOLTAGE	6				//Contrast



	//A0 Port
	#define PORT_A0  			PORTD
	#define PIN_A0_bm  			0x02

	//Reset Port
	#define PORT_RST 			PORTD
	#define PIN_RST_bm 			0x01

	//SPI Port
	#define SPI_PORT			PORTD
			


	//SPI-Befehle
	#define CMD		1							// write to lc-display command or data register --> command
	#define DATA	0							// write to lc-display command or data register --> data
	#define DISPLAY_ON       			0xAF  	// LCD_DISPLAY_ON
	#define DISPLAY_OFF      			0xAE  	// LCD_DISPLAY_OFF
	#define DISPLAY_PAGE_ADDRESS		0xB0	// (3) Page address set
	#define DISPLAY_COL_ADDRESS_MSB		0x10	// (4) Column address set upper bit
	#define DISPLAY_COL_ADDRESS_LSB		0x00  	// (4) Column address set lower bit
	#define DISPLAY_BOTTOMVIEW			0xA0 	// (8) ADC select
	#define DISPLAY_TOPVIEW				0xA1  
	#define DISPLAY_NORMAL   			0xA6	// (9) Display normale/reverse
	#define DISPLAY_REVERSE				0xA7
	#define DISPLAY_SHOW_NORMAL			0xA4	// (10) Display all points ON/OFF
	#define DISPLAY_SHOW_ALL_POINTS		0xA5
	#define DISPLAY_BIAS_1_9			0xA2	// (11) LCD bias set
	#define DISPLAY_BIAS_1_7			0xA3
	#define DISPLAY_RESET_CMD			0xE2	// (14) RESET
	#define DISPLAY_SCAN_DIR_NORMAL		0xC0  	// (15) Common output mode select
	#define DISPLAY_SCAN_DIR_REVERSE	0xC8  
	#define DISPLAY_POWER_CONTROL		0x28	// (16) Power control set
	#define DISPLAY_POWER_LOW_POWER		0x2F
	#define DISPLAY_POWER_WIDE_RANGE	0x2F 
	#define DISPLAY_POWER_LOW_VOLTAGE	0x2B 
	#define DISPLAY_VOLTAGE          	0x20 	// (17) V0 voltage regulator internal resistor ratio set// + volume controll
	#define DISPLAY_VOLUME_MODE_1    	0x81	// (18) Electronic volume mode set
	#define DISPLAY_VOLUME_MODE_2    	0x00	// (18) Register
	#define DISPLAY_INDICATOR_ON       	0xAD  	// (19) Static indicator ON/OFF
	#define DISPLAY_INDICATOR_OFF      	0xAC  
	#define DISPLAY_INDICATOR_MODE_OFF 	0x00	// (19) Static indicator register set
	#define DISPLAY_INDICATOR_MODE_1HZ 	0x01
	#define DISPLAY_INDICATOR_MODE_2HZ 	0x10
	#define DISPLAY_INDICATOR_MODE_ON  	0x11
	#define DISPLAY_BOOSTER_SET      	0xF8	// (20) Booster ratio set
	#define DISPLAY_BOOSTER_234      	0x00
	#define DISPLAY_BOOSTER_5        	0x01
	#define DISPLAY_BOOSTER_6        	0x03
	#define LCD_NOP              		0xE3	// (22) NOP
	

	

	// Funktionen bekanntmachen
	void Lcd_SetPixel(uint8_t x, uint8_t y);
	void Lcd_Show(void);
        void Lcd_Clear(void);
        void Lcd_Hline(uint8_t x, uint8_t y, uint8_t w);
	void display_init(void);
	void display_clear(void);
	void display_go_to (unsigned char, unsigned char);
						
#endif //_DISPLAY_H






