/*----------------------------------------------------------------------------
 Project:      		Testprojekt fr Grafikdisplay EA DOG L ber SPI
 Author:         	Falko Jahn
------------------------------------------------------------------------------*/

#include <avr/interrupt.h>
#include <avr/io.h>
#include <avr/pgmspace.h>

#include "grafik_display.h"
#include "spi_driver.h"





void CCPIOWrite(volatile uint8_t * address, uint8_t value) {
   volatile uint8_t * tmpAddr = address;
   
   asm volatile(
      "movw r30,  %0"   "\n\t"
      "ldi  r16,  %2"   "\n\t"
      "out   %3, r16"   "\n\t"
      "st     Z,  %1"
      :
      : "r" (tmpAddr), "r" (value), "M" (CCP_IOREG_gc), "m" (CCP)
      : "r16", "r30", "r31"
      );
} 




// ---------- CONFIG SYSTEM CLOCK ------------------------------------------------------//
// 
void ConfigClockSystem( void )
{
	/* Start external Oscillator  */
	/* Start internal 32MHz RC oscillator. */
	OSC.CTRL = OSC_RC32MEN_bm;

	do {
		/* Wait while oscillator stabilizes. */
	} while ( ( OSC.STATUS & OSC_RC32MRDY_bm ) == 0 );

	//  Setup and enable the PLL
   OSC.PLLCTRL = OSC_PLLSRC_RC32M_gc | 16;
   OSC.CTRL     |= OSC_PLLEN_bm;
   while ((OSC.STATUS & OSC_PLLRDY_bm) == 0);
	

	
	//  Setup the clock prescalers
    //  Note this operation is protected by the CCP register
    CCPIOWrite(&CLK.PSCTRL, (CLK_PSADIV_1_gc | CLK_PSBCDIV_2_2_gc));

    //  Switch to the PLL as the clock source
    CCPIOWrite(&CLK.CTRL, CLK_SCLKSEL_PLL_gc);

}




//############################################################################
//Hauptprogramm
int main (void) 
//############################################################################
{
	//initialisiere Clock
	ConfigClockSystem();

	//initialisiere Display
	display_init();


	
	//Display wird gelscht
	Lcd_Clear();	
	

	//setPixel();
	Lcd_SetPixel(0,0);

	//draw line
	Lcd_Hline(3,3,20);
	Lcd_Hline(2,5,25);
	Lcd_Show();
	


	while(1)
	{
	}
}


