/* 
 * Prototype test 004
 * ------------------
 * 
 * The FTDI232RL USB connection will be tested. The program just echos
 * all received data back to the sender. Additionally the received 
 * character will be assigned to a port (PORTA by default) for debugging
 * purposes (e.g. connect the LEDs).
 *
 * The ATXMega128A1 will be clocked to 80 MHz with 40 MHz peripherical 
 * clock. A connected 8 MHz oscillator will be assumed.
 *
 * Connect at least pin 1-2 and 3-4 on JP_FT232. Additionally connect
 * JP_LED to PORTA.
 *
 * The serial line will be configured with 19.200 baud 8N1.
 * 
 * For configuration see configuration.h header.
 * 
 * Thomas Koos, 10.2009
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdbool.h>

#include "configuration.h"
#include "clock.h"
#include "serial.h"

  // init1() prototype declaration
void init1 (void) __attribute__ ((naked)) __attribute__ ((section (".init1")));

/*
 * configures system clocks and usart
 */
void init1(void)
{
  configure_clock();
  configure_usart();
}

/*
 * main program
 */
int main(void)
{
    // configure LED port if configured
#ifdef LEDPORT
    // set all pins to output
  LEDPORT.DIR = 0xff;   
    // set all pins to high (LED off, LED are low active)
  LEDPORT.OUT = 0xff;
#endif

    // enable interrupts
  sei();

    // do until power off...
  while (true)
  {
      // something received?
    if (USARTFlag)
    {
        // output to LED port if configured
#ifdef LEDPORT
      LEDPORT.OUT = USARTChar;
#endif

        // echo received data
      usart_send(USARTChar);
        // clear received flag
      USARTFlag = false;
    }
  }

  return 0;
};







