/*
 * Prototype test 004
 * ------------------
 * 
 * clock configuration
 *
 */

#include "clock.h"
#include "CCPIO.h"
#include <avr/io.h>

/*
 * Sets up the system clock.
 *
 * parameters:
 *   clock source       external oscillator, 8 MHz
 *   PLL settings       x4 (32 MHz)
 *   prescaler          A: 1 (32 MHz)
 *
 */
void configure_clock(void) 
{
      //  1. Enable the external clock and wait for it to be stable

      // external OSC between 2 and 9 MHz
      // external OSC needs 16K CLKs to startup
    OSC.XOSCCTRL  = OSC_FRQRANGE_2TO9_gc | OSC_XOSCSEL_XTAL_16KCLK_gc;
      // enable external OSC
    OSC.CTRL     |= OSC_XOSCEN_bm;
      // wait till stable
    while ((OSC.STATUS & OSC_XOSCRDY_bm) == 0);

      //  2. Setup and enable the PLL

      // select external OSC as source 
      // multiplicator is 4 (resulting in 32 MHz)
    OSC.PLLCTRL = OSC_PLLSRC_XOSC_gc | (0x04 << OSC_PLLFAC_gp);
      // enable PLL
    OSC.CTRL   |= OSC_PLLEN_bm;
    while ((OSC.STATUS & OSC_PLLRDY_bm) == 0);

      // 3. Setup the clock prescalers
    
      // Note this operation is protected by the CCP register
      // clock divider: 1
    CCPIOWrite(&CLK.PSCTRL, (CLK_PSADIV_1_gc | CLK_PSBCDIV_1_1_gc));

      // Switch to the PLL as the clock source
    CCPIOWrite(&CLK.CTRL, CLK_SCLKSEL_PLL_gc);

      // RTCEN (0x01) RTC clock enable
      // TOSC32 32 kHz from 32 kHz Crystal OSC on TOSC
    CLK.RTCCTRL = CLK_RTCSRC_TOSC32_gc | 0x01;

    //  Lock the setting cown
    //CLK.LOCK = CLK_LOCK_bm; 
}
