/*
 * Prototype test 004
 * ------------------
 * 
 * serial communications
 *
 */

#include "serial.h"
#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdint.h>

volatile char USARTChar = 0x00;
volatile bool USARTFlag = false;

/*
 * Configurate the USART 1 on port F for communication using
 * asynchronous serial transfer with 8N1 on 19200 baud.
 *
 * This initialisation requires a Fper of 40 MHz!
 */
void configure_usart(void)
{
    // set TxD to high
  PORTF.OUT = 0x40;   // taken from AVR docs

    // set TxD & RTS to output
  PORTF.DIRSET = PIN7_bm | PIN4_bm;
    // set RxD & CTS to input
  PORTF.DIRCLR = PIN6_bm | PIN5_bm;

    // set up USART

    // receive interrupt level medium
  USARTF1.CTRLA = USART_RXCINTLVL_LO_gc;
    // enable TxD and RxD
  USARTF1.CTRLB = USART_RXEN_bm | USART_TXEN_bm;
    // asynchronous mode, parity none, one stop bit, 8 bits per char
  USARTF1.CTRLC = USART_CMODE_ASYNCHRONOUS_gc | USART_PMODE_DISABLED_gc | USART_CHSIZE_8BIT_gc;

    // Baudrate calculated with aid of the Atmel supplied Excel sheet
    // CPU F = 32 MHz
    // Rate = 19200 Baud
    // CLK2X = false
    //
    // BSEL = 103 (0x81)
    // Error = 0.16%
    // 
  USARTF1.BAUDCTRLA = 0x67;
    // no scale factor
  USARTF1.BAUDCTRLB = 0x00;
    
    // enable low level interrupts in PIC
  PMIC.CTRL |= PMIC_LOLVLEX_bm;
}

/*
 * send data on USART 1 on port F
 */
void usart_send(uint8_t data)
{
    // is the transmit buffer empty (ready to receive transmission data)?
  while (!(USARTF1.STATUS & USART_DREIF_bm)) 
    ;

    // write data to transmission buffer
  USARTF1.DATA = data;
}

/*
 * USART 1 on port F receive handler ISR
 */
ISR(USARTF1_RXC_vect)
{
  USARTChar = USARTF1.DATA;
  USARTFlag = true;
}



