/* 
 * Prototype test 005
 * ------------------
 * The rotary encoder will be tested.
 *
 * NOTE: In event mode the directions are swapped.I couldn't fix it.
 *
 * Turning clockwise increases value shown by leds.
 * Turning counter-clockwise decreases value shown by leds.
 * Pushing the button makes the shown value flash.
 * Value is also displayed by UART.
 * Connect JP_LED to the appropriate port header in order to test it.
 * Connect Jumpers of the rotary encoder (JP_ENC).
 * Choose event based implemention by defining EVENT
 * Choose timer based implemention by not defining EVENT 
 * Also tests the external crystal oscillator and the internal PLL to bring
 * the system up to 32MHz.
 *
 * -------------
 *
 * (cl) Alexander Frank, 03.2010
 */

// sources
// (Timer based software implemention) rotary encoder mikrocontroller.net: http://www.mikrocontroller.net/articles/Drehgeber#Solide_L.C3.B6sung:_Beispielcode_in_C
// (Event based implemention) check alternative: AVR1600 Quadrature Decoder. The Timer is used to let the LEDs blink
// Timer Atxmega:http://www.stromflo.de/dokuwiki/doku.php?id=xmega-c-tutorial#timer_mit_interrupt

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "usart.h"//USART from 006_sdcard

#define EVENT 1//Decoder Implemention 0:Timer 1:Event
#define FREQ 32000000UL
#define PORT_LED PORTC 

#define PORT_ENC_BTN PORTE
#define PORT_ENC_A PORTE
#define PORT_ENC_B PORTE

#define ENC_BTN PIN0_bp
#define ENC_A PIN2_bp//QDPH90
#define ENC_B PIN1_bp//QDPH0

#define PHASE_A	(PORT_ENC_A.IN & (1<<ENC_A))
#define PHASE_B	(PORT_ENC_B.IN & (1<<ENC_B))
#define BTN_PRESSED (~PORT_ENC_BTN.IN & (1<<ENC_BTN))

void configure_clock(void);
void CCPIOWrite(volatile uint8_t * address, uint8_t value); 
void init1 (void) __attribute__ ((naked)) __attribute__ ((section (".init1")));
void timer_init (void);
void port_init (void);
void encode_init (void);
void quadenc_init (void);
int8_t encode_read4 (void);

volatile int8_t enc_delta; // -128 ... 127
static int8_t last;
volatile uint8_t led_on;

int main(void)
{
  timer_init();
  uart_init();
  sei();
#ifdef EVENT
  quadenc_init();
#else
  port_init();
  encode_init();
#endif

  led_on=0;
  uart_puts("Date:"__DATE__);
  uart_puts(" Time:"__TIME__);
  uart_eol();

  while (1)
    {
      static uint8_t val = 0;
      static uint8_t val_last = 100;
      static uint8_t send_once=0;
#ifdef EVENT
      val =(uint8_t)(TCE0.CNT>>2);
#else
      val += encode_read4(); // read a single step encoder
#endif
      if (!led_on && BTN_PRESSED)
	{
	  PORT_LED.OUT = 0xFF;
	}
      else
	{
	  PORT_LED.OUT = ~val;
	}
      if (val != val_last)
	{
	  uart_putdecbyte (val);
	  uart_puts(" ");
	  uart_putbinbyte (val);
	  uart_eol();
	}
      val_last=val;

      if (!BTN_PRESSED)
	{
	  send_once = 0;
	}
      if (BTN_PRESSED && !send_once)
	{
	  send_once = !send_once;
	  uart_puts ("Pressed");
	  uart_eol();
	}
    }
  return 0;
}

ISR(TCC0_OVF_vect)
{
  static uint16_t ms=0;
#ifndef EVENT
  int8_t new, diff; 
  new = 0;
  if( PHASE_A )
    new = 3;
  if( PHASE_B )
    new ^= 1;					// convert gray to binary
  diff = last - new;				// difference last - new
  if( diff & 1 ){				// bit 0 = value (1)
    last = new;					// store new as next last
    enc_delta += (diff & 2) - 1;		// bit 1 = direction (+/-)
  }
#endif  
  if(ms++==100) // every 100ms
    {
      ms=0;
      led_on=!led_on;// toggle on/off state
    }
}

void timer_init (void)
{
  PMIC.CTRL |= (1<<PIN0)|(1<<PIN1)|(1<<PIN2);
  TCC0.PER = (uint8_t)(FREQ / 1024 * 1e-3 -0.5); // 1ms  
  TCC0.CTRLA = TC_CLKSEL_DIV1024_gc;
  TCC0.CTRLB = 0;
  TCC0.INTCTRLA = TC_OVFINTLVL_HI_gc; // Overflow Interrupt: Level High Level(3)
}

void port_init (void)
{
  PORT_LED.DIR = 0xff; // all leds outputs
  PORT_LED.OUT = 0xff; // all leds off
  PORT_ENC_A.DIR = 0x00; // encoder pins inputs
  //enable pullups
  /* MPCMASK register is cleared automatically after the write operation to the pin configuration registers is finished. */ 
  PORTCFG.MPCMASK = 0x7;
  PORT_ENC_A.PIN0CTRL = ( PORT_ENC_A.PIN0CTRL & ~PORT_OPC_gm ) |
    PORT_OPC_PULLUP_gc ; 
}

void encode_init( void )
{
  int8_t new;
 
  new = 0;
  if( PHASE_A )
    new = 3;
  if( PHASE_B )
    new ^= 1;					// convert gray to binary
  last = new;					// power on state
  enc_delta = 0;
}
 
int8_t encode_read4( void )			// read four step encoders
{
  int8_t val;
 
  cli();
  val = enc_delta;
  enc_delta = val & 3;
  sei();
  return val >> 2;
}

void quadenc_init (void)
{
  PORT_LED.DIR = 0xff; // all leds outputs
  PORT_LED.OUT = 0xff; // all leds off
  PORT_ENC_A.DIR = 0x00; // encoder pins inputs 1.)
  // QDEC_Port_Setup
  /* Set QDPH0 and QDPH1 sensing level. */
  PORTCFG.MPCMASK = (1<<ENC_A)|(1<<ENC_B);//pin 1&2
  PORT_ENC_B.PIN0CTRL = (PORT_ENC_B.PIN0CTRL & ~PORT_ISC_gm) | PORT_ISC_LEVEL_gc | PORT_OPC_PULLUP_gc; //2.) Input/Sense Configuration|Sense Level (Transparent For Events)|Totempole w/ Pull-up on Input
  PORTCFG.MPCMASK = (1<<ENC_BTN);//pin 0 btn
  PORT_ENC_BTN.PIN0CTRL = PORT_ENC_BTN.PIN0CTRL | PORT_OPC_PULLUP_gc ;//pullup btn
  // QDEC_EVSYS_Setup
  EVSYS.CH0MUX = EVSYS_CHMUX_PORTE_PIN1_gc;//3.)
  EVSYS.CH0CTRL = EVSYS_QDEN_bm | EVSYS_DIGFILT_2SAMPLES_gc;//5.)//enable quadrature decode|digital filter coefficient=2 DS p.73  
  // QDEC_TC_Dec_Setup
  TCE0.CTRLD = (uint8_t) TC_EVACT_QDEC_gc | TC_EVSEL_CH0_gc;//6.) 7.)
  TCE0.PER = (0xFF+1)*4;//8.)
  TCE0.CTRLA = TC_CLKSEL_DIV1_gc;//9.)
} 

void init1(void)
{
  configure_clock();
}

/*
 * Sets up the system clock.
 *
 * parameters:
 *   clock source       external oscillator, 8 MHz
 *   PLL settings       x4 (32 MHz)
 *   prescaler          A: 1 (32 MHz)
 *
 */
void configure_clock(void) 
{
  //  1. Enable the external clock and wait for it to be stable

  // external OSC between 2 and 9 MHz
  // external OSC needs 16K CLKs to startup
  OSC.XOSCCTRL  = OSC_FRQRANGE_2TO9_gc | OSC_XOSCSEL_XTAL_16KCLK_gc;
  // enable external OSC
  OSC.CTRL     |= OSC_XOSCEN_bm;
  // wait till stable
  while ((OSC.STATUS & OSC_XOSCRDY_bm) == 0);

  //  2. Setup and enable the PLL

  // select external OSC as source 
  // multiplicator is 4 (resulting in 32 MHz)
  OSC.PLLCTRL = OSC_PLLSRC_XOSC_gc | (0x04 << OSC_PLLFAC_gp);
  // enable PLL
  OSC.CTRL   |= OSC_PLLEN_bm;
  while ((OSC.STATUS & OSC_PLLRDY_bm) == 0);

  // 3. Setup the clock prescalers
    
  // Note this operation is protected by the CCP register
  // clock divider: 1
  CCPIOWrite(&CLK.PSCTRL, (CLK_PSADIV_1_gc | CLK_PSBCDIV_1_1_gc));

  // Switch to the PLL as the clock source
  CCPIOWrite(&CLK.CTRL, CLK_SCLKSEL_PLL_gc);

  // RTCEN (0x01) RTC clock enable
  // TOSC32 32 kHz from 32 kHz Crystal OSC on TOSC
  CLK.RTCCTRL = CLK_RTCSRC_TOSC32_gc | 0x01;

  //  Lock the setting cown
  //CLK.LOCK = CLK_LOCK_bm; 
}

void CCPIOWrite(volatile uint8_t * address, uint8_t value) {
  volatile uint8_t * tmpAddr = address;
   
  asm volatile(
	       "movw r30,  %0"   "\n\t"
	       "ldi  r16,  %2"   "\n\t"
	       "out   %3, r16"   "\n\t"
	       "st     Z,  %1"
	       :
	       : "r" (tmpAddr), "r" (value), "M" (CCP_IOREG_gc), "m" (CCP)
	       : "r16", "r30", "r31"
	       );
} 
