#include <avr/io.h>
#include <util/delay.h>
#include "usart.h"


void uart_init(void)
{
//    PORTF.OUT=0x08;
//    PORTF.DIR=0x08;
    PORTF.OUT=0x80;
    PORTF.DIR=0x80;

    // ATTENTION: This was intended to be used with a 8MHz
    //            crystal (x4 = 32MHz).
    // see http://www.avrcalc.elektronik-projekt.de/xmega/baud_rate_calculator

    /* set baud rate 38400Baud@32MHz */
    USARTF1_BAUDCTRLA= 0x33;  /* high byte */
    USARTF1_BAUDCTRLB= 0x00;  /* low byte */

    /* enable transmitter and receiver */
    USARTF1_CTRLB = _BV(USART_TXEN_bp) | _BV(USART_RXEN_bp);

} 

void  uart_putc(uint8_t data)
{

    while (!(USARTF1_STATUS & _BV(USART_DREIF_bp)));

    USARTF1_DATA = data;

}

void uart_putf(void *buffer)
{

    uint8_t c;
    uint8_t *b = (uint8_t *)buffer;

    while ((c = pgm_read_byte(b)) != '\0') {
        uart_putc(c);
        b++;
    }

} 


void uart_puts(char *buffer)
{

    uint8_t *b = (uint8_t *)buffer;

    while (*b != '\0') {
        uart_putc(*b);
        b++;
    }

}

void uart_puthexnibble(uint8_t data)
{

    if (data > 9) {
        data -= 10;
        uart_putc('A' + data);
    } else {
        uart_putc('0' + data);
    }

} 

void uart_puthexbyte(uint8_t data)
{

    /* first high, then low nibble */
    uart_puthexnibble((data & 0xF0) >> 4);
    uart_puthexnibble(data & 0x0F);

}

void  uart_putdecbyte(uint8_t data)
{

    uint8_t pot10[] = {100, 10};

    for (uint8_t i=0; i<2; i++) {

        uint16_t p = pot10[i];
        uint8_t digit = 48;

        while (data >= p) {
            digit++;
            data -= p;
        }

        uart_putc(digit);
    }
    uart_putc(data+48);
}

void uart_putbinbyte (uint8_t data)
{
  if (0 == data) {
    return;
  } else {
    uart_putbinbyte(data/2);
    uart_putc((data % 2) ? '1' : '0');
  }
}

void uart_eol(void)
{
    uart_puts_P("\r\n");
}

