#include <avr/io.h>
#include <util/delay.h>


#include "config.h"

#ifdef USARTDEBUG
#include "usart.h"
#endif

void spi_init(void)
{

    /* configure MOSI, SCK lines as outputs */
    //
    // SPI_SS will be set to output 
    // It can be used for User-I/O (as output)
    // Using it as Input would disturb SPI-Master-Mode
    //
    PORTE.DIR |= 0xB0;
    

    /* enable spi, set master and clock modes (f/8) */
	SPIE_CTRL = _BV(SPI_MASTER_bp);
	SPIE_CTRL|= _BV(SPI_ENABLE_bp) |
	            _BV(SPI_CLK2X_bp)  |
	            _BV(SPI_PRESCALER0_bp);

}

void spi_wait_busy(void)
{

#ifdef SPI_TIMEOUT
    uint8_t timeout = 200;

    while (!(SPIE_STATUS & _BV(SPI_IF_bp)) && timeout > 0)
        timeout--;

#ifdef USARTDEBUG
    if (timeout == 0)
        uart_puts_P("ERROR: spi timeout reached!\r\n");
#endif
#else
    while (!(SPIE_STATUS & _BV(SPI_IF_bp)));
    {
          asm volatile ("nop");
    }
#endif

}

uint8_t spi_send(uint8_t data)
{

    SPIE_DATA = data;
    spi_wait_busy();

    return SPIE_DATA;

}

void SpiWriteChunk(     const uint8_t* pChunk,
                                        uint16_t nBytes)
{
        while (nBytes--)
        {
                SPIE_DATA = *pChunk++;
				while (!(SPIE_STATUS & _BV(SPI_IF_bp)))
                {
                        asm volatile ("nop");
                }
        }
}

void SpiReadChunk(      uint8_t* pChunk,
                                        uint16_t nBytes)
{
        while (nBytes--)
        {
                SPIE_DATA = 0;
				while (!(SPIE_STATUS & _BV(SPI_IF_bp)))
                {
                        asm volatile ("nop");
                }

                *pChunk++ = SPIE_DATA;
        }

}
