/* This is a program stub (skelleton) to be used with the Atxmega128a1 starter board.
 * You might use this stub to start your own projects. It provides the basic infrastructure
 * and a high quality Makefile for AVR GCC.
 *
 * (cl) Michel Gross, 06.2009
 */

#include <avr/io.h>

void configure_clock(void);
void CCPIOWrite(volatile uint8_t * address, uint8_t value); 
void init1 (void) __attribute__ ((naked)) __attribute__ ((section (".init1")));

void init1(void)
{
  configure_clock();
}

int main(void)
{
  // code here 

  return 0;
}

/*
 * Sets up the system clock.
 */
void configure_clock(void) {
    //  Enable the external clock and wait for it to be stable
    OSC.XOSCCTRL  = OSC_FRQRANGE_2TO9_gc | OSC_XOSCSEL_XTAL_16KCLK_gc;
    OSC.CTRL     |= OSC_XOSCEN_bm;
    while ((OSC.STATUS & OSC_XOSCRDY_bm) == 0);

    //  Setup and enable the PLL (4*8 = 32MHz)
    OSC.PLLCTRL = OSC_PLLSRC_XOSC_gc | (0x04 << OSC_PLLFAC_gp);
    OSC.CTRL     |= OSC_PLLEN_bm;
    while ((OSC.STATUS & OSC_PLLRDY_bm) == 0);

    //  Setup the clock prescalers
    //  Note this operation is protected by the CCP register
    CCPIOWrite(&CLK.PSCTRL, (CLK_PSADIV_1_gc | CLK_PSBCDIV_1_1_gc));

    //  Switch to the PLL as the clock source
    CCPIOWrite(&CLK.CTRL, CLK_SCLKSEL_PLL_gc);

    //  Lock the setting cown
    //CLK.LOCK = CLK_LOCK_bm; 
}

void CCPIOWrite(volatile uint8_t * address, uint8_t value) {
   volatile uint8_t * tmpAddr = address;
   
   asm volatile(
      "movw r30,  %0"   "\n\t"
      "ldi  r16,  %2"   "\n\t"
      "out   %3, r16"   "\n\t"
      "st     Z,  %1"
      :
      : "r" (tmpAddr), "r" (value), "M" (CCP_IOREG_gc), "m" (CCP)
      : "r16", "r30", "r31");
} 
